"""Definition of XAudioAPI - requests that user can make to the device."""

from typing import List, Optional, Tuple

from xaudio.clients import XAudioClient
from xaudio.protocol.interface_pb2 import (  # pylint:disable=no-name-in-module
    A2BDiscoverRequest,
    A2BMailboxAccessType,
    A2BMailboxTransferRequest,
    A2BMailboxTransferResponse,
    I2COverDistanceAccessType,
    I2COverDistanceRequest,
    I2COverDistanceResponse,
    InfoRequest,
    InfoResponse,
    NoDataResponse,
    RequestPacket,
    ResetRequest,
    StatusRequest,
    StatusResponse,
)


class XAudioApi:
    """XAudio available API."""

    def __init__(self, client: XAudioClient):
        self.client = client

    def info(self) -> InfoResponse:
        """Get device info.

        :return: device details like software/hardware revision or serial number

        """
        info_request = InfoRequest(dummy=True)
        request_packet = RequestPacket(info_request=info_request)
        response = self.client.request(request_packet)
        return response

    def reset(self) -> NoDataResponse:
        """Reset device - device responds before reset is performed.

        :return: confirmation of request before reset

        """
        reset_request = ResetRequest(dummy=True)
        request_packet = RequestPacket(reset_request=reset_request)
        response = self.client.request(request_packet)
        return response

    def status(self) -> StatusResponse:
        """Get device status info.

        :return: look at `StatusResponse` for more details

        """
        status_request = StatusRequest(dummy=True)
        request_packet = RequestPacket(status_request=status_request)
        response = self.client.request(request_packet)
        return response

    def a2b_discover(self) -> NoDataResponse:
        """Rediscover A2B bus.

        :return: confirmation of request

        """
        a2b_discover_request = A2BDiscoverRequest(dummy=True)
        request_packet = RequestPacket(a2b_discover_request=a2b_discover_request)
        response = self.client.request(request_packet)
        return response

    @staticmethod
    def _parse_i2c_data_to_send(
        data: List[Tuple[int, Optional[int]]],
    ) -> List[I2COverDistanceRequest.Data]:
        """Parse pairs of registries and values into I2COverDistanceRequest.Data structure."""
        parsed_data = []
        for reg, val in [(t[0], t[1]) if len(t) == 2 else (t[0], None) for t in data]:
            parsed_data.append(I2COverDistanceRequest.Data(reg=reg, value=val))
        return parsed_data

    def i2c_over_distance(
        self,
        node: int,
        access_type: "I2COverDistanceAccessType",
        data: List[Tuple[int, Optional[int]]],  #  "List[I2COverDistanceRequest.Data]",
        peripheral_i2c_addr: Optional[int] = None,
    ) -> I2COverDistanceResponse:
        """Send I2C message to a node or its peripherals.

        Used to read/write data from/to node (or its peripherals) registries over I2C.
        Look at the example below to read data from slave node registries:

            >>> client = XAudioClient("COM5")
            >>> to_send = [(0x01, 0x02), (0x02, None), (0x03,)]
            >>> read = I2COverDistanceAccessType.I2C_OVER_DISTANCE_READ  # or write
            >>> api = XAudioApi(client)
            >>> resp = api.i2c_over_distance(
            ...     node=0,
            ...     access_type=read,
            ...     data=to_send
            ...   # peripheral_i2c_addr=None,  # address of the peripheral device if used
            ... )
            >>> print(resp)

        And to write the data back to slave node registries:

            >>> client = XAudioClient("COM5")
            >>> to_send = [(0x01, 0xFF), (0x02, 0x00), (0x03, 0xAB)]
            >>> read = I2COverDistanceAccessType.I2C_OVER_DISTANCE_READ  # or write
            >>> api = XAudioApi(client)
            >>> resp = api.i2c_over_distance(
            ...     node=0,
            ...     access_type=read,
            ...     data=to_send
            ...   # peripheral_i2c_addr=None,  # address of the peripheral device if used
            ... )
            >>> print(resp)

        :param node: number, specified in the json configuration
        :param access_type: read/write/unspecified
        :param data: list of pairs (registry, value) to read/write,
            value is ignored on read
        :param peripheral_i2c_addr: if not given the operation will be performed
            directly on node and not its peripherals
        :return: access type and reg values

        """
        data = self._parse_i2c_data_to_send(data)
        i2c_over_distance_request = I2COverDistanceRequest(
            access_type=access_type,
            peripheral_i2c_addr=peripheral_i2c_addr,
            node=node,
            data=data,
        )
        request_packet = RequestPacket(
            i2c_over_distance_request=i2c_over_distance_request
        )
        response = self.client.request(request_packet)
        return response

    def a2b_mailbox_transfer(
        self,
        node: int,
        mailbox_id: int,
        access_type: A2BMailboxAccessType,
        _bytes: Optional[int] = None,
        data: Optional[list[int]] = None,
    ) -> A2BMailboxTransferResponse:
        """Send/read mailbox data to/from A2B transceiver.

        Sample use:

            >>> client = XAudioClient("COM5")
            >>> to_send = [0x00, 0x01, 0x02, 0x03]  # 4 bytes
            >>> write = A2BMailboxAccessType.A2B_MAILBOX_ACCESS_TYPE_WRITE
            >>> api = XAudioApi(client)
            >>> resp = api.a2b_mailbox_transfer(
            ...     node=0,
            ...     access_type=write,
            ...     data=to_send,
            ...   # _bytes=4,  # len(data) required on read, auto calculated on write
            ... )
            >>> print(resp)

        :param node: id from json configuration file (Slave)
        :param mailbox_id: slave mailbox id from config file
        :param access_type: read/write/unspecified
        :param _bytes: number of bytes to read or write, on write it's len(data)
        :param data: list of bytes to send
        :return: refer to A2BMailboxTransferResponse fields

        """
        if data is None:
            data = []
        a2b_mailbox_transfer_request = A2BMailboxTransferRequest(
            mailbox_id=mailbox_id,
            access_type=access_type,
            node=node,
            bytes=_bytes or len(data),
            data=data,
        )
        request_packet = RequestPacket(
            a2b_mailbox_transfer_request=a2b_mailbox_transfer_request
        )
        response = self.client.request(request_packet)
        return response



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html
# pylint: skip-file
from datetime import date

from pkg_resources import get_distribution

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = "xaudio"
copyright = f"{date.today().year}, int2code"
author = "int2code"
version = release = get_distribution("xaudio").version

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "sphinx.ext.duration",
    "sphinx.ext.doctest",
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinxcontrib.programoutput",
    "myst_parser",
]
autosummary_generate = True

# uncomment when _templates won't be empty
# templates_path = ['._templates']
exclude_patterns = [
    "_build",
    "Thumbs.db",
    ".DS_Store",
    "README.md",  # this is readme describing how to generate docs
]


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "sphinx_rtd_theme"
# uncomment when static content is added
# html_static_path = ['_static']

# suppress warning from release notes and readme markdown files
suppress_warnings = ["myst.header"]



"""Quick sample use of XAudio Client and API."""

# pylint:disable=no-name-in-module

from xaudio.api import XAudioApi
from xaudio.clients import XAudioClient
from xaudio.protocol.interface_pb2 import InfoRequest, RequestPacket

# Replace with actual COM port where XAudio is connected
COM_PORT = "COM99"
client = XAudioClient(COM_PORT, f"XAudio_on_{COM_PORT}")

# Send sample request with the generic request
# from XAudio client -> each request must return
# single response msg even when calling reset
sample_msg = RequestPacket(info_request=InfoRequest(dummy=True)).SerializeToString()
response = client.request(sample_msg)
print(response)


# Convenient API client for sending and reading messages
# with simplified message building
api_client = XAudioApi(client)
response = api_client.info()
print(response.serial_number)
print(response.hardware_revision)
print(response.software_revision)

response = api_client.status()
print(response)



"""XAudio defined exceptions and errors."""


class XAudioException(Exception):
    """Base class for other XAudio exceptions."""


class XAudioTimeoutError(XAudioException):
    """Raised when XAudio did not respond in expected time."""


class XAudioResponseError(XAudioException):
    """Unexpected response detected."""



"""Test XAudio"""

from contextlib import contextmanager
from unittest.mock import ANY, patch

import pytest
from device_communication.serial.servers import ThreadedSerialServer

from xaudio.api import XAudioApi
from xaudio.clients import XAudioClient
from xaudio.packetizer import XAudioFramesParser
from xaudio.protocol.interface_pb2 import (  # pylint:disable=no-name-in-module
    A2BDiscoverRequest,
    A2BFaultLocation,
    A2BMailboxAccessStatus,
    A2BMailboxAccessType,
    A2BMailboxTransferRequest,
    A2BMailboxTransferResponse,
    ConfigJsonState,
    DeviceState,
    I2COverDistanceAccessType,
    I2COverDistanceRequest,
    I2COverDistanceResponse,
    InfoRequest,
    InfoResponse,
    NoDataResponse,
    PositiveResponse,
    RequestPacket,
    ResetRequest,
    ResponsePacket,
    SlaveA2BState,
    StatusRequest,
    StatusResponse,
    StatusRespRoleA2BMaster,
    StatusRespRoleA2BSlave,
    UsbAudioStreamState,
)


class TestXAudioApi:
    """Verify XAudio API responses"""

    @contextmanager
    def patch_serial_on_write_read(self, raw_response):
        """Patch write method of server to trigger reading data."""

        # pylint:disable=unused-argument
        def _trigger_read_on_write(self, data):  # noqa protocol.write API
            self.protocol.data_received(raw_response)

        with patch.object(ThreadedSerialServer, "write", autospec=True) as mock:
            mock.side_effect = _trigger_read_on_write
            yield mock

    @pytest.fixture
    def xaudio_api(self) -> XAudioApi:
        """XAudioApi instance"""
        client = XAudioClient()
        return XAudioApi(client)

    def test_info(self, xaudio_api):
        """Verify info request sending and unpacking."""
        expected_request = (
            ANY,
            XAudioFramesParser.build_frame(
                RequestPacket(info_request=InfoRequest(dummy=True)).SerializeToString()
            ),
        )
        expected_response = InfoResponse(
            hardware_revision=1, software_revision="2", serial_number="3"
        )
        raw_response = XAudioFramesParser.build_frame(
            ResponsePacket(
                positive_response=PositiveResponse(info_response=expected_response)
            ).SerializeToString()
        )
        with self.patch_serial_on_write_read(raw_response) as mock_rw:
            response = xaudio_api.info()

        assert response == expected_response
        mock_rw.assert_called_once_with(*expected_request)

    def test_reset(self, xaudio_api):
        """Verify reset request sending and unpacking."""
        expected_request = (
            ANY,
            XAudioFramesParser.build_frame(
                RequestPacket(
                    reset_request=ResetRequest(dummy=True)
                ).SerializeToString()
            ),
        )
        expected_response = NoDataResponse(dummy=True)
        raw_response = XAudioFramesParser.build_frame(
            ResponsePacket(
                positive_response=PositiveResponse(no_data_response=expected_response)
            ).SerializeToString()
        )
        with self.patch_serial_on_write_read(raw_response) as mock_rw:
            response = xaudio_api.reset()

        assert response == expected_response
        mock_rw.assert_called_once_with(*expected_request)

    def test_status(self, xaudio_api):
        """Verify reset request sending and unpacking."""
        expected_request = (
            ANY,
            XAudioFramesParser.build_frame(
                RequestPacket(
                    status_request=StatusRequest(dummy=True)
                ).SerializeToString()
            ),
        )
        expected_response = StatusResponse(
            usb_audio_downstream_state=UsbAudioStreamState.USB_AUDIO_STREAM_STATE_IDLE,
            usb_audio_upstream_state=UsbAudioStreamState.USB_AUDIO_STREAM_STATE_UNSPECIFIED,
            device_state=DeviceState.DEVICE_STATE_NORMAL,
            config_json_state=ConfigJsonState.CONFIG_JSON_STATE_VALID,
            status_master=StatusRespRoleA2BMaster(
                a2b_slaves_discovered=1,
                a2b_fault=StatusRespRoleA2BMaster.A2bFault(
                    fault=1,
                    location=A2BFaultLocation.A2B_FAULT_LOCATION_UNSPECIFIED,
                    slave_with_fault=2,
                ),
            ),
            status_slave=StatusRespRoleA2BSlave(
                a2b_state=SlaveA2BState.SLAVE_A2B_STATE_UNSPECIFIED
            ),
        )
        raw_response = XAudioFramesParser.build_frame(
            ResponsePacket(
                positive_response=PositiveResponse(status_response=expected_response)
            ).SerializeToString()
        )
        with self.patch_serial_on_write_read(raw_response) as mock_rw:
            response = xaudio_api.status()

        assert response == expected_response
        mock_rw.assert_called_once_with(*expected_request)

    def test_a2b_discover(self, xaudio_api):
        """Verify reset request sending and unpacking."""
        expected_request = (
            ANY,
            XAudioFramesParser.build_frame(
                RequestPacket(
                    a2b_discover_request=A2BDiscoverRequest(dummy=True)
                ).SerializeToString()
            ),
        )
        expected_response = NoDataResponse(dummy=True)
        raw_response = XAudioFramesParser.build_frame(
            ResponsePacket(
                positive_response=PositiveResponse(no_data_response=expected_response)
            ).SerializeToString()
        )
        with self.patch_serial_on_write_read(raw_response) as mock_rw:
            response = xaudio_api.a2b_discover()

        assert response == expected_response
        mock_rw.assert_called_once_with(*expected_request)

    def test_i2c_over_distance(self, xaudio_api):
        """Verify i2c_over_distance request sending and unpacking."""
        expected_request = (
            ANY,
            XAudioFramesParser.build_frame(
                RequestPacket(
                    i2c_over_distance_request=I2COverDistanceRequest(
                        access_type=I2COverDistanceAccessType.I2C_OVER_DISTANCE_READ,
                        peripheral_i2c_addr=1,
                        node=2,
                        data=[
                            I2COverDistanceRequest.Data(reg=8, value=9),
                            I2COverDistanceRequest.Data(reg=10, value=None),
                            I2COverDistanceRequest.Data(reg=11, value=None),
                        ],
                    )
                ).SerializeToString()
            ),
        )
        expected_response = I2COverDistanceResponse(value=[9])
        raw_response = XAudioFramesParser.build_frame(
            ResponsePacket(
                positive_response=PositiveResponse(
                    i2c_over_distance_response=expected_response
                )
            ).SerializeToString()
        )
        with self.patch_serial_on_write_read(raw_response) as mock_rw:
            response = xaudio_api.i2c_over_distance(
                access_type=I2COverDistanceAccessType.I2C_OVER_DISTANCE_READ,
                peripheral_i2c_addr=1,
                node=2,
                data=[(8, 9), (10, None), (11,)],
            )

        assert response == expected_response
        mock_rw.assert_called_once_with(*expected_request)

    def test_a2b_mailbox_transfer(self, xaudio_api):
        """Verify a2b_mailbox_transfer request sending and unpacking."""
        expected_request = (
            ANY,
            XAudioFramesParser.build_frame(
                RequestPacket(
                    a2b_mailbox_transfer_request=A2BMailboxTransferRequest(
                        mailbox_id=0,
                        access_type=A2BMailboxAccessType.A2B_MAILBOX_ACCESS_TYPE_WRITE,
                        node=0,
                        bytes=4,
                        data=[0x00, 0x01, 0x02, 0x03],
                    )
                ).SerializeToString()
            ),
        )
        expected_response = A2BMailboxTransferResponse(
            access_type=A2BMailboxAccessType.A2B_MAILBOX_ACCESS_TYPE_WRITE,
            access_status=A2BMailboxAccessStatus.A2B_MAILBOX_STATUS_OK,
            data=[0x00, 0x01, 0x02, 0x03],
        )
        raw_response = XAudioFramesParser.build_frame(
            ResponsePacket(
                positive_response=PositiveResponse(
                    a2b_mailbox_transfer_response=expected_response
                )
            ).SerializeToString()
        )
        with self.patch_serial_on_write_read(raw_response) as mock_rw:
            response = xaudio_api.a2b_mailbox_transfer(
                node=0,
                mailbox_id=0,
                access_type=A2BMailboxAccessType.A2B_MAILBOX_ACCESS_TYPE_WRITE,
                data=[0x00, 0x01, 0x02, 0x03],
            )

        assert response == expected_response
        mock_rw.assert_called_once_with(*expected_request)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: interface.proto
# Protobuf Python Version: 6.30.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC, 6, 30, 0, "", "interface.proto"
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import xaudio.protocol.nanopb_pb2 as nanopb__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x0finterface.proto\x1a\x0cnanopb.proto",\n\x0cResetRequest\x12\x12\n\x05\x64ummy\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x08\n\x06_dummy"-\n\rStatusRequest\x12\x12\n\x05\x64ummy\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x08\n\x06_dummy"2\n\x12\x41\x32\x42\x44iscoverRequest\x12\x12\n\x05\x64ummy\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x08\n\x06_dummy"+\n\x0bInfoRequest\x12\x12\n\x05\x64ummy\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x08\n\x06_dummy"E\n\x10SetSerialRequest\x12\x15\n\rserial_number\x18\x01 \x01(\t\x12\x11\n\x04lock\x18\x02 \x01(\x08H\x00\x88\x01\x01\x42\x07\n\x05_lock"\xe8\x01\n\x16I2COverDistanceRequest\x12/\n\x0b\x61\x63\x63\x65ss_type\x18\x01 \x01(\x0e\x32\x1a.I2COverDistanceAccessType\x12 \n\x13peripheral_i2c_addr\x18\x02 \x01(\rH\x00\x88\x01\x01\x12\x0c\n\x04node\x18\x03 \x01(\r\x12\x31\n\x04\x64\x61ta\x18\x06 \x03(\x0b\x32\x1c.I2COverDistanceRequest.DataB\x05\x92?\x02\x10\n\x1a"\n\x04\x44\x61ta\x12\x0b\n\x03reg\x18\x04 \x01(\r\x12\r\n\x05value\x18\x05 \x01(\rB\x16\n\x14_peripheral_i2c_addr"`\n\x17I2COverDistanceResponse\x12/\n\x0b\x61\x63\x63\x65ss_type\x18\x01 \x01(\x0e\x32\x1a.I2COverDistanceAccessType\x12\x14\n\x05value\x18\x02 \x03(\rB\x05\x92?\x02\x10\n"\x8d\x01\n\x19\x41\x32\x42MailboxTransferRequest\x12\x12\n\nmailbox_id\x18\x01 \x01(\r\x12*\n\x0b\x61\x63\x63\x65ss_type\x18\x02 \x01(\x0e\x32\x15.A2BMailboxAccessType\x12\x0c\n\x04node\x18\x03 \x01(\r\x12\r\n\x05\x62ytes\x18\x04 \x01(\r\x12\x13\n\x04\x64\x61ta\x18\x05 \x03(\rB\x05\x92?\x02\x10\x04"\xa1\x01\n\x1a\x41\x32\x42MailboxTransferResponse\x12\x12\n\nmailbox_id\x18\x01 \x01(\r\x12*\n\x0b\x61\x63\x63\x65ss_type\x18\x02 \x01(\x0e\x32\x15.A2BMailboxAccessType\x12.\n\raccess_status\x18\x03 \x01(\x0e\x32\x17.A2BMailboxAccessStatus\x12\x13\n\x04\x64\x61ta\x18\x04 \x03(\rB\x05\x92?\x02\x10\x04".\n\x0eNoDataResponse\x12\x12\n\x05\x64ummy\x18\x01 \x01(\x08H\x00\x88\x01\x01\x42\x08\n\x06_dummy"\xf5\x01\n\x17StatusRespRoleA2BMaster\x12\x1d\n\x15\x61\x32\x62_slaves_discovered\x18\x01 \x01(\r\x12\x39\n\ta2b_fault\x18\x02 \x01(\x0b\x32!.StatusRespRoleA2BMaster.A2bFaultH\x00\x88\x01\x01\x1ar\n\x08\x41\x32\x62\x46\x61ult\x12\r\n\x05\x66\x61ult\x18\x01 \x01(\r\x12#\n\x08location\x18\x02 \x01(\x0e\x32\x11.A2BFaultLocation\x12\x1d\n\x10slave_with_fault\x18\x03 \x01(\rH\x00\x88\x01\x01\x42\x13\n\x11_slave_with_faultB\x0c\n\n_a2b_fault";\n\x16StatusRespRoleA2BSlave\x12!\n\ta2b_state\x18\x01 \x01(\x0e\x32\x0e.SlaveA2BState"\xc3\x02\n\x0eStatusResponse\x12\x38\n\x1ausb_audio_downstream_state\x18\x01 \x01(\x0e\x32\x14.UsbAudioStreamState\x12\x36\n\x18usb_audio_upstream_state\x18\x02 \x01(\x0e\x32\x14.UsbAudioStreamState\x12"\n\x0c\x64\x65vice_state\x18\x03 \x01(\x0e\x32\x0c.DeviceState\x12+\n\x11\x63onfig_json_state\x18\x04 \x01(\x0e\x32\x10.ConfigJsonState\x12\x31\n\rstatus_master\x18\x05 \x01(\x0b\x32\x18.StatusRespRoleA2BMasterH\x00\x12/\n\x0cstatus_slave\x18\x06 \x01(\x0b\x32\x17.StatusRespRoleA2BSlaveH\x00\x42\n\n\x08oneofmsg"[\n\x0cInfoResponse\x12\x19\n\x11hardware_revision\x18\x01 \x01(\r\x12\x19\n\x11software_revision\x18\x02 \x01(\t\x12\x15\n\rserial_number\x18\x03 \x01(\t"\xa5\x02\n\x10PositiveResponse\x12+\n\x10no_data_response\x18\x01 \x01(\x0b\x32\x0f.NoDataResponseH\x00\x12*\n\x0fstatus_response\x18\x02 \x01(\x0b\x32\x0f.StatusResponseH\x00\x12&\n\rinfo_response\x18\x03 \x01(\x0b\x32\r.InfoResponseH\x00\x12>\n\x1ai2c_over_distance_response\x18\x04 \x01(\x0b\x32\x18.I2COverDistanceResponseH\x00\x12\x44\n\x1d\x61\x32\x62_mailbox_transfer_response\x18\x05 \x01(\x0b\x32\x1b.A2BMailboxTransferResponseH\x00\x42\n\n\x08oneofmsg"G\n\x10NegativeResponse\x12\x11\n\x07no_data\x18\x01 \x01(\x08H\x00\x12\x14\n\ntext_error\x18\x02 \x01(\tH\x00\x42\n\n\x08oneofmsg"\xfb\x02\n\rRequestPacket\x12&\n\rreset_request\x18\x01 \x01(\x0b\x32\r.ResetRequestH\x00\x12\x33\n\x14\x61\x32\x62_discover_request\x18\x02 \x01(\x0b\x32\x13.A2BDiscoverRequestH\x00\x12(\n\x0estatus_request\x18\x03 \x01(\x0b\x32\x0e.StatusRequestH\x00\x12$\n\x0cinfo_request\x18\x04 \x01(\x0b\x32\x0c.InfoRequestH\x00\x12/\n\x12set_serial_request\x18\x05 \x01(\x0b\x32\x11.SetSerialRequestH\x00\x12<\n\x19i2c_over_distance_request\x18\x06 \x01(\x0b\x32\x17.I2COverDistanceRequestH\x00\x12\x42\n\x1c\x61\x32\x62_mailbox_transfer_request\x18\x07 \x01(\x0b\x32\x1a.A2BMailboxTransferRequestH\x00\x42\n\n\x08oneofmsg"|\n\x0eResponsePacket\x12.\n\x11positive_response\x18\x01 \x01(\x0b\x32\x11.PositiveResponseH\x00\x12.\n\x11negative_response\x18\x02 \x01(\x0b\x32\x11.NegativeResponseH\x00\x42\n\n\x08oneofmsg*s\n\x10\x41\x32\x42\x46\x61ultLocation\x12"\n\x1e\x41\x32\x42_FAULT_LOCATION_UNSPECIFIED\x10\x00\x12\x1d\n\x19\x41\x32\x42_FAULT_LOCATION_MASTER\x10\x01\x12\x1c\n\x18\x41\x32\x42_FAULT_LOCATION_SLAVE\x10\x02*\xa7\x01\n\rSlaveA2BState\x12\x1f\n\x1bSLAVE_A2B_STATE_UNSPECIFIED\x10\x00\x12\x18\n\x14SLAVE_A2B_STATE_INIT\x10\x01\x12!\n\x1dSLAVE_A2B_STATE_WAIT_DISCOVER\x10\x02\x12\x19\n\x15SLAVE_A2B_STATE_READY\x10\x03\x12\x1d\n\x19SLAVE_A2B_STATE_NOT_READY\x10\x04*\x84\x01\n\x13UsbAudioStreamState\x12&\n"USB_AUDIO_STREAM_STATE_UNSPECIFIED\x10\x00\x12\x1f\n\x1bUSB_AUDIO_STREAM_STATE_IDLE\x10\x01\x12$\n USB_AUDIO_STREAM_STATE_STREAMING\x10\x02*\x8e\x01\n\x0b\x44\x65viceState\x12\x1c\n\x18\x44\x45VICE_STATE_UNSPECIFIED\x10\x00\x12\x15\n\x11\x44\x45VICE_STATE_BOOT\x10\x01\x12\x17\n\x13\x44\x45VICE_STATE_NORMAL\x10\x02\x12\x19\n\x15\x44\x45VICE_STATE_IMPAIRED\x10\x03\x12\x16\n\x12\x44\x45VICE_STATE_ERROR\x10\x04*w\n\x19I2COverDistanceAccessType\x12!\n\x1dI2C_OVER_DISTANCE_UNSPECIFIED\x10\x00\x12\x1b\n\x17I2C_OVER_DISTANCE_WRITE\x10\x01\x12\x1a\n\x16I2C_OVER_DISTANCE_READ\x10\x02*p\n\x0f\x43onfigJsonState\x12!\n\x1d\x43ONFIG_JSON_STATE_UNSPECIFIED\x10\x00\x12\x1b\n\x17\x43ONFIG_JSON_STATE_VALID\x10\x01\x12\x1d\n\x19\x43ONFIG_JSON_STATE_INVALID\x10\x02*\x84\x01\n\x14\x41\x32\x42MailboxAccessType\x12\'\n#A2B_MAILBOX_ACCESS_TYPE_UNSPECIFIED\x10\x00\x12!\n\x1d\x41\x32\x42_MAILBOX_ACCESS_TYPE_WRITE\x10\x01\x12 \n\x1c\x41\x32\x42_MAILBOX_ACCESS_TYPE_READ\x10\x02*\xbf\x01\n\x16\x41\x32\x42MailboxAccessStatus\x12"\n\x1e\x41\x32\x42_MAILBOX_STATUS_UNSPECIFIED\x10\x00\x12\x19\n\x15\x41\x32\x42_MAILBOX_STATUS_OK\x10\x01\x12#\n\x1f\x41\x32\x42_MAILBOX_STATUS_GENERAL_FAIL\x10\x02\x12 \n\x1c\x41\x32\x42_MAILBOX_STATUS_NOT_EMPTY\x10\x03\x12\x1f\n\x1b\x41\x32\x42_MAILBOX_STATUS_NOT_FULL\x10\x04\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "interface_pb2", _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals["_I2COVERDISTANCEREQUEST"].fields_by_name["data"]._loaded_options = None
    _globals["_I2COVERDISTANCEREQUEST"].fields_by_name[
        "data"
    ]._serialized_options = b"\222?\002\020\n"
    _globals["_I2COVERDISTANCERESPONSE"].fields_by_name["value"]._loaded_options = None
    _globals["_I2COVERDISTANCERESPONSE"].fields_by_name[
        "value"
    ]._serialized_options = b"\222?\002\020\n"
    _globals["_A2BMAILBOXTRANSFERREQUEST"].fields_by_name["data"]._loaded_options = None
    _globals["_A2BMAILBOXTRANSFERREQUEST"].fields_by_name[
        "data"
    ]._serialized_options = b"\222?\002\020\004"
    _globals["_A2BMAILBOXTRANSFERRESPONSE"].fields_by_name[
        "data"
    ]._loaded_options = None
    _globals["_A2BMAILBOXTRANSFERRESPONSE"].fields_by_name[
        "data"
    ]._serialized_options = b"\222?\002\020\004"
    _globals["_A2BFAULTLOCATION"]._serialized_start = 2588
    _globals["_A2BFAULTLOCATION"]._serialized_end = 2703
    _globals["_SLAVEA2BSTATE"]._serialized_start = 2706
    _globals["_SLAVEA2BSTATE"]._serialized_end = 2873
    _globals["_USBAUDIOSTREAMSTATE"]._serialized_start = 2876
    _globals["_USBAUDIOSTREAMSTATE"]._serialized_end = 3008
    _globals["_DEVICESTATE"]._serialized_start = 3011
    _globals["_DEVICESTATE"]._serialized_end = 3153
    _globals["_I2COVERDISTANCEACCESSTYPE"]._serialized_start = 3155
    _globals["_I2COVERDISTANCEACCESSTYPE"]._serialized_end = 3274
    _globals["_CONFIGJSONSTATE"]._serialized_start = 3276
    _globals["_CONFIGJSONSTATE"]._serialized_end = 3388
    _globals["_A2BMAILBOXACCESSTYPE"]._serialized_start = 3391
    _globals["_A2BMAILBOXACCESSTYPE"]._serialized_end = 3523
    _globals["_A2BMAILBOXACCESSSTATUS"]._serialized_start = 3526
    _globals["_A2BMAILBOXACCESSSTATUS"]._serialized_end = 3717
    _globals["_RESETREQUEST"]._serialized_start = 33
    _globals["_RESETREQUEST"]._serialized_end = 77
    _globals["_STATUSREQUEST"]._serialized_start = 79
    _globals["_STATUSREQUEST"]._serialized_end = 124
    _globals["_A2BDISCOVERREQUEST"]._serialized_start = 126
    _globals["_A2BDISCOVERREQUEST"]._serialized_end = 176
    _globals["_INFOREQUEST"]._serialized_start = 178
    _globals["_INFOREQUEST"]._serialized_end = 221
    _globals["_SETSERIALREQUEST"]._serialized_start = 223
    _globals["_SETSERIALREQUEST"]._serialized_end = 292
    _globals["_I2COVERDISTANCEREQUEST"]._serialized_start = 295
    _globals["_I2COVERDISTANCEREQUEST"]._serialized_end = 527
    _globals["_I2COVERDISTANCEREQUEST_DATA"]._serialized_start = 469
    _globals["_I2COVERDISTANCEREQUEST_DATA"]._serialized_end = 503
    _globals["_I2COVERDISTANCERESPONSE"]._serialized_start = 529
    _globals["_I2COVERDISTANCERESPONSE"]._serialized_end = 625
    _globals["_A2BMAILBOXTRANSFERREQUEST"]._serialized_start = 628
    _globals["_A2BMAILBOXTRANSFERREQUEST"]._serialized_end = 769
    _globals["_A2BMAILBOXTRANSFERRESPONSE"]._serialized_start = 772
    _globals["_A2BMAILBOXTRANSFERRESPONSE"]._serialized_end = 933
    _globals["_NODATARESPONSE"]._serialized_start = 935
    _globals["_NODATARESPONSE"]._serialized_end = 981
    _globals["_STATUSRESPROLEA2BMASTER"]._serialized_start = 984
    _globals["_STATUSRESPROLEA2BMASTER"]._serialized_end = 1229
    _globals["_STATUSRESPROLEA2BMASTER_A2BFAULT"]._serialized_start = 1101
    _globals["_STATUSRESPROLEA2BMASTER_A2BFAULT"]._serialized_end = 1215
    _globals["_STATUSRESPROLEA2BSLAVE"]._serialized_start = 1231
    _globals["_STATUSRESPROLEA2BSLAVE"]._serialized_end = 1290
    _globals["_STATUSRESPONSE"]._serialized_start = 1293
    _globals["_STATUSRESPONSE"]._serialized_end = 1616
    _globals["_INFORESPONSE"]._serialized_start = 1618
    _globals["_INFORESPONSE"]._serialized_end = 1709
    _globals["_POSITIVERESPONSE"]._serialized_start = 1712
    _globals["_POSITIVERESPONSE"]._serialized_end = 2005
    _globals["_NEGATIVERESPONSE"]._serialized_start = 2007
    _globals["_NEGATIVERESPONSE"]._serialized_end = 2078
    _globals["_REQUESTPACKET"]._serialized_start = 2081
    _globals["_REQUESTPACKET"]._serialized_end = 2460
    _globals["_RESPONSEPACKET"]._serialized_start = 2462
    _globals["_RESPONSEPACKET"]._serialized_end = 2586
# @@protoc_insertion_point(module_scope)



"""Test XAudioClients"""

from unittest.mock import DEFAULT, Mock, patch

import pytest

from xaudio.clients import XAudioClient
from xaudio.communication_handlers import XAudioSerialCommHandler
from xaudio.exceptions import XAudioResponseError
from xaudio.protocol.interface_pb2 import (  # pylint:disable=no-name-in-module
    NoDataResponse,
    RequestPacket,
)


class TestXAudioClient:
    """Verify XAudioClient functionality"""

    def test_too_many_responses_for_request(self):
        """Verify exception is raised if more than one response msg is returned"""
        with patch.multiple(
            XAudioSerialCommHandler, send=DEFAULT, receive=Mock(return_value=[b""] * 2)
        ):
            client = XAudioClient()
            with pytest.raises(XAudioResponseError, match="Too many responses"):
                client.request(RequestPacket())

    def test_response_no_data_packet(self):
        """Verify no data packet is returned on empty bytes."""
        with patch.multiple(
            XAudioSerialCommHandler, send=DEFAULT, receive=Mock(return_value=[b""])
        ):
            client = XAudioClient()
            response = client.request(RequestPacket())
            assert isinstance(response, NoDataResponse)

    def test_negative_response_raise_exception(self):
        """Verify exception is raised when device sends negative response"""
        with patch.multiple(
            XAudioSerialCommHandler,
            send=DEFAULT,
            receive=Mock(return_value=[b"\x12\x02\x08\x01"]),
        ):
            client = XAudioClient()
            with pytest.raises(XAudioResponseError, match="returned negative response"):
                client.request(RequestPacket())



"""Clients implementation to send/read requests/responses to the device."""

import time
from typing import Union

from xaudio.communication_handlers import XAudioSerialCommHandler
from xaudio.exceptions import XAudioResponseError, XAudioTimeoutError
from xaudio.protocol.interface_pb2 import (  # pylint:disable=no-name-in-module
    A2BMailboxTransferResponse,
    I2COverDistanceResponse,
    InfoResponse,
    NegativeResponse,
    NoDataResponse,
    RequestPacket,
    ResponsePacket,
    StatusResponse,
)

OneOfPositiveResponseMsg = Union[
    NoDataResponse,
    StatusResponse,
    InfoResponse,
    I2COverDistanceResponse,
    A2BMailboxTransferResponse,
]


class XAudioClient:  # pylint:disable=too-few-public-methods
    """XAudio client with generic request implementation."""

    def __init__(
        self,
        port_name: str = "loop://",
        name: str = "XAudioHandler",
        timeout: int | float = 2,
    ):
        """Initialize instance.

        :param port_name: for serial (i.e. COM2)
        :param name: of communication handler for distinction
        :param timeout: time in s to wait for response

        """
        self.comm_handler = XAudioSerialCommHandler(port_name, 115200, 2, name)
        self.comm_handler.make_connection()
        self._timeout = timeout

    def request(self, data: RequestPacket) -> OneOfPositiveResponseMsg:
        """Send RequestPacket to target over communication handler and wait for response.

        :param data: to send
        :return: response msg form device

        """
        self.comm_handler.send(data.SerializeToString())

        start = time.time()
        while time.time() - start < self._timeout:
            responses = list(self.comm_handler.receive())
            if responses:
                break
        else:
            raise XAudioTimeoutError(f"No response in expected time for\n{data}")

        if len(responses) > 1:
            raise XAudioResponseError(f"Too many responses from device: {responses}")

        if responses[0] == b"":
            return NoDataResponse()

        rp = ResponsePacket.FromString(responses[0])
        msg_name = rp.WhichOneof("oneofmsg")
        if not msg_name:
            raise XAudioResponseError(
                "ResponsePacket is missing Positive or Negative msg, "
                f"packet content: {responses[0]}"
            )

        positive_response = getattr(rp, msg_name)
        if isinstance(positive_response, NegativeResponse):
            raise XAudioResponseError(
                f"Request returned negative response {positive_response}"
            )

        return getattr(positive_response, positive_response.WhichOneof("oneofmsg"))



"""Module provides classes to read/write binary data."""

import logging
from typing import List, Tuple

from cobs import cobs
from device_communication.base.packetizer import BaseFrameParser
from device_communication.serial.servers import ThreadedSerialServer
from libscrc import mpeg2  # pylint:disable=no-name-in-module
from serial.threaded import Packetizer

logger = logging.getLogger(__name__)


class XAudioFramesParser(BaseFrameParser):
    """XAudio frame parser."""

    TERMINATOR = b"\x00"
    CRC_SIZE = 4

    @classmethod
    def build_frame(cls, data: bytearray) -> bytearray:
        """Frame data buffer

        Calculate 4B CRC32/MPEG2 and encode whole buffer with COBS - end with terminator.

        :param data: buffer to send
        :return: encoded data with CRC and terminator

        """
        return (
            cobs.encode(data + mpeg2(data).to_bytes(cls.CRC_SIZE, byteorder="little"))
            + cls.TERMINATOR
        )

    @classmethod
    def parse_frames(
        cls, buffer: bytearray
    ) -> Tuple[List[bytearray], bytearray, bytearray]:
        """Parse given buffer to frames

        :param buffer: incoming data stream
        :returns:
            parsed XAudio frames found in buffer,
            remaining buffer,
            dropped data (CRC did not match)

        """
        packets = []
        dropped = bytearray()
        while cls.TERMINATOR in buffer:
            packet, buffer = buffer.split(cls.TERMINATOR, 1)
            packet = cobs.decode(packet)
            packet, crc = packet[: -cls.CRC_SIZE], packet[-cls.CRC_SIZE :]
            if not crc == mpeg2(packet).to_bytes(cls.CRC_SIZE, byteorder="little"):
                dropped.extend(packet + crc)
            else:
                packets.append(packet)
        return packets, buffer, dropped


class XAudioPacketizer(Packetizer):
    """Read/write data from/to serial port.

    The COBS encoding with CRC32/MPEG2 (4B) and terminator are applied.

    """

    FRAME_PARSER_CLS = XAudioFramesParser

    def __init__(self):
        super().__init__()
        self.transport: ThreadedSerialServer = None  # noqa

    def connection_made(self, transport):
        """Store transport"""
        self.transport = transport

    def connection_lost(self, exc):
        """Forget transport"""
        self.transport = None
        super(Packetizer, self).connection_lost(exc)

    def data_received(self, data):
        """Buffer received data, find TERMINATOR, call handle_packet"""
        self.buffer.extend(data)
        packets, self.buffer, dropped = self.FRAME_PARSER_CLS.parse_frames(self.buffer)

        for packet in packets:
            self.handle_packet(packet)

        self.handle_dropped_packet(dropped)

    def handle_packet(self, packet: bytearray):
        """Process packet - store in queue for client to read"""
        self.transport.incoming_buffer.put_nowait(packet)

    def handle_dropped_packet(self, packet: bytearray):
        """Process dropped packets - store in queue"""
        logger.debug("Could not parse data: %s", packet.hex(" "))

    def send(self, data: bytearray):
        """Prepare packet and send it to transport for write"""
        self.transport.write(self.FRAME_PARSER_CLS.build_frame(data))



"""Communication handlers for XAudio"""

from device_communication.serial.communication_handlers import (
    SerialCommunicationHandler,
)

from xaudio.packetizer import XAudioPacketizer


class XAudioSerialCommHandler(SerialCommunicationHandler):
    """Serial communication handler with XAudio protocol class."""

    SERIAL_PROTOCOL_CLS = XAudioPacketizer



from collections.abc import Iterable as _Iterable
from collections.abc import Mapping as _Mapping
from typing import ClassVar as _ClassVar
from typing import Optional as _Optional
from typing import Union as _Union

import nanopb_pb2 as _nanopb_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper

DESCRIPTOR: _descriptor.FileDescriptor

class A2BFaultLocation(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    A2B_FAULT_LOCATION_UNSPECIFIED: _ClassVar[A2BFaultLocation]
    A2B_FAULT_LOCATION_MASTER: _ClassVar[A2BFaultLocation]
    A2B_FAULT_LOCATION_SLAVE: _ClassVar[A2BFaultLocation]

class SlaveA2BState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    SLAVE_A2B_STATE_UNSPECIFIED: _ClassVar[SlaveA2BState]
    SLAVE_A2B_STATE_INIT: _ClassVar[SlaveA2BState]
    SLAVE_A2B_STATE_WAIT_DISCOVER: _ClassVar[SlaveA2BState]
    SLAVE_A2B_STATE_READY: _ClassVar[SlaveA2BState]
    SLAVE_A2B_STATE_NOT_READY: _ClassVar[SlaveA2BState]

class UsbAudioStreamState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    USB_AUDIO_STREAM_STATE_UNSPECIFIED: _ClassVar[UsbAudioStreamState]
    USB_AUDIO_STREAM_STATE_IDLE: _ClassVar[UsbAudioStreamState]
    USB_AUDIO_STREAM_STATE_STREAMING: _ClassVar[UsbAudioStreamState]

class DeviceState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    DEVICE_STATE_UNSPECIFIED: _ClassVar[DeviceState]
    DEVICE_STATE_BOOT: _ClassVar[DeviceState]
    DEVICE_STATE_NORMAL: _ClassVar[DeviceState]
    DEVICE_STATE_IMPAIRED: _ClassVar[DeviceState]
    DEVICE_STATE_ERROR: _ClassVar[DeviceState]

class I2COverDistanceAccessType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    I2C_OVER_DISTANCE_UNSPECIFIED: _ClassVar[I2COverDistanceAccessType]
    I2C_OVER_DISTANCE_WRITE: _ClassVar[I2COverDistanceAccessType]
    I2C_OVER_DISTANCE_READ: _ClassVar[I2COverDistanceAccessType]

class ConfigJsonState(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    CONFIG_JSON_STATE_UNSPECIFIED: _ClassVar[ConfigJsonState]
    CONFIG_JSON_STATE_VALID: _ClassVar[ConfigJsonState]
    CONFIG_JSON_STATE_INVALID: _ClassVar[ConfigJsonState]

class A2BMailboxAccessType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    A2B_MAILBOX_ACCESS_TYPE_UNSPECIFIED: _ClassVar[A2BMailboxAccessType]
    A2B_MAILBOX_ACCESS_TYPE_WRITE: _ClassVar[A2BMailboxAccessType]
    A2B_MAILBOX_ACCESS_TYPE_READ: _ClassVar[A2BMailboxAccessType]

class A2BMailboxAccessStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    A2B_MAILBOX_STATUS_UNSPECIFIED: _ClassVar[A2BMailboxAccessStatus]
    A2B_MAILBOX_STATUS_OK: _ClassVar[A2BMailboxAccessStatus]
    A2B_MAILBOX_STATUS_GENERAL_FAIL: _ClassVar[A2BMailboxAccessStatus]
    A2B_MAILBOX_STATUS_NOT_EMPTY: _ClassVar[A2BMailboxAccessStatus]
    A2B_MAILBOX_STATUS_NOT_FULL: _ClassVar[A2BMailboxAccessStatus]

A2B_FAULT_LOCATION_UNSPECIFIED: A2BFaultLocation
A2B_FAULT_LOCATION_MASTER: A2BFaultLocation
A2B_FAULT_LOCATION_SLAVE: A2BFaultLocation
SLAVE_A2B_STATE_UNSPECIFIED: SlaveA2BState
SLAVE_A2B_STATE_INIT: SlaveA2BState
SLAVE_A2B_STATE_WAIT_DISCOVER: SlaveA2BState
SLAVE_A2B_STATE_READY: SlaveA2BState
SLAVE_A2B_STATE_NOT_READY: SlaveA2BState
USB_AUDIO_STREAM_STATE_UNSPECIFIED: UsbAudioStreamState
USB_AUDIO_STREAM_STATE_IDLE: UsbAudioStreamState
USB_AUDIO_STREAM_STATE_STREAMING: UsbAudioStreamState
DEVICE_STATE_UNSPECIFIED: DeviceState
DEVICE_STATE_BOOT: DeviceState
DEVICE_STATE_NORMAL: DeviceState
DEVICE_STATE_IMPAIRED: DeviceState
DEVICE_STATE_ERROR: DeviceState
I2C_OVER_DISTANCE_UNSPECIFIED: I2COverDistanceAccessType
I2C_OVER_DISTANCE_WRITE: I2COverDistanceAccessType
I2C_OVER_DISTANCE_READ: I2COverDistanceAccessType
CONFIG_JSON_STATE_UNSPECIFIED: ConfigJsonState
CONFIG_JSON_STATE_VALID: ConfigJsonState
CONFIG_JSON_STATE_INVALID: ConfigJsonState
A2B_MAILBOX_ACCESS_TYPE_UNSPECIFIED: A2BMailboxAccessType
A2B_MAILBOX_ACCESS_TYPE_WRITE: A2BMailboxAccessType
A2B_MAILBOX_ACCESS_TYPE_READ: A2BMailboxAccessType
A2B_MAILBOX_STATUS_UNSPECIFIED: A2BMailboxAccessStatus
A2B_MAILBOX_STATUS_OK: A2BMailboxAccessStatus
A2B_MAILBOX_STATUS_GENERAL_FAIL: A2BMailboxAccessStatus
A2B_MAILBOX_STATUS_NOT_EMPTY: A2BMailboxAccessStatus
A2B_MAILBOX_STATUS_NOT_FULL: A2BMailboxAccessStatus

class ResetRequest(_message.Message):
    __slots__ = ("dummy",)
    DUMMY_FIELD_NUMBER: _ClassVar[int]
    dummy: bool
    def __init__(self, dummy: bool = ...) -> None: ...

class StatusRequest(_message.Message):
    __slots__ = ("dummy",)
    DUMMY_FIELD_NUMBER: _ClassVar[int]
    dummy: bool
    def __init__(self, dummy: bool = ...) -> None: ...

class A2BDiscoverRequest(_message.Message):
    __slots__ = ("dummy",)
    DUMMY_FIELD_NUMBER: _ClassVar[int]
    dummy: bool
    def __init__(self, dummy: bool = ...) -> None: ...

class InfoRequest(_message.Message):
    __slots__ = ("dummy",)
    DUMMY_FIELD_NUMBER: _ClassVar[int]
    dummy: bool
    def __init__(self, dummy: bool = ...) -> None: ...

class SetSerialRequest(_message.Message):
    __slots__ = ("serial_number", "lock")
    SERIAL_NUMBER_FIELD_NUMBER: _ClassVar[int]
    LOCK_FIELD_NUMBER: _ClassVar[int]
    serial_number: str
    lock: bool
    def __init__(
        self, serial_number: _Optional[str] = ..., lock: bool = ...
    ) -> None: ...

class I2COverDistanceRequest(_message.Message):
    __slots__ = ("access_type", "peripheral_i2c_addr", "node", "data")

    class Data(_message.Message):
        __slots__ = ("reg", "value")
        REG_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        reg: int
        value: int
        def __init__(
            self, reg: _Optional[int] = ..., value: _Optional[int] = ...
        ) -> None: ...

    ACCESS_TYPE_FIELD_NUMBER: _ClassVar[int]
    PERIPHERAL_I2C_ADDR_FIELD_NUMBER: _ClassVar[int]
    NODE_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    access_type: I2COverDistanceAccessType
    peripheral_i2c_addr: int
    node: int
    data: _containers.RepeatedCompositeFieldContainer[I2COverDistanceRequest.Data]
    def __init__(
        self,
        access_type: _Optional[_Union[I2COverDistanceAccessType, str]] = ...,
        peripheral_i2c_addr: _Optional[int] = ...,
        node: _Optional[int] = ...,
        data: _Optional[_Iterable[_Union[I2COverDistanceRequest.Data, _Mapping]]] = ...,
    ) -> None: ...

class I2COverDistanceResponse(_message.Message):
    __slots__ = ("access_type", "value")
    ACCESS_TYPE_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    access_type: I2COverDistanceAccessType
    value: _containers.RepeatedScalarFieldContainer[int]
    def __init__(
        self,
        access_type: _Optional[_Union[I2COverDistanceAccessType, str]] = ...,
        value: _Optional[_Iterable[int]] = ...,
    ) -> None: ...

class A2BMailboxTransferRequest(_message.Message):
    __slots__ = ("mailbox_id", "access_type", "node", "bytes", "data")
    MAILBOX_ID_FIELD_NUMBER: _ClassVar[int]
    ACCESS_TYPE_FIELD_NUMBER: _ClassVar[int]
    NODE_FIELD_NUMBER: _ClassVar[int]
    BYTES_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    mailbox_id: int
    access_type: A2BMailboxAccessType
    node: int
    bytes: int
    data: _containers.RepeatedScalarFieldContainer[int]
    def __init__(
        self,
        mailbox_id: _Optional[int] = ...,
        access_type: _Optional[_Union[A2BMailboxAccessType, str]] = ...,
        node: _Optional[int] = ...,
        bytes: _Optional[int] = ...,
        data: _Optional[_Iterable[int]] = ...,
    ) -> None: ...

class A2BMailboxTransferResponse(_message.Message):
    __slots__ = ("mailbox_id", "access_type", "access_status", "data")
    MAILBOX_ID_FIELD_NUMBER: _ClassVar[int]
    ACCESS_TYPE_FIELD_NUMBER: _ClassVar[int]
    ACCESS_STATUS_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    mailbox_id: int
    access_type: A2BMailboxAccessType
    access_status: A2BMailboxAccessStatus
    data: _containers.RepeatedScalarFieldContainer[int]
    def __init__(
        self,
        mailbox_id: _Optional[int] = ...,
        access_type: _Optional[_Union[A2BMailboxAccessType, str]] = ...,
        access_status: _Optional[_Union[A2BMailboxAccessStatus, str]] = ...,
        data: _Optional[_Iterable[int]] = ...,
    ) -> None: ...

class NoDataResponse(_message.Message):
    __slots__ = ("dummy",)
    DUMMY_FIELD_NUMBER: _ClassVar[int]
    dummy: bool
    def __init__(self, dummy: bool = ...) -> None: ...

class StatusRespRoleA2BMaster(_message.Message):
    __slots__ = ("a2b_slaves_discovered", "a2b_fault")

    class A2bFault(_message.Message):
        __slots__ = ("fault", "location", "slave_with_fault")
        FAULT_FIELD_NUMBER: _ClassVar[int]
        LOCATION_FIELD_NUMBER: _ClassVar[int]
        SLAVE_WITH_FAULT_FIELD_NUMBER: _ClassVar[int]
        fault: int
        location: A2BFaultLocation
        slave_with_fault: int
        def __init__(
            self,
            fault: _Optional[int] = ...,
            location: _Optional[_Union[A2BFaultLocation, str]] = ...,
            slave_with_fault: _Optional[int] = ...,
        ) -> None: ...

    A2B_SLAVES_DISCOVERED_FIELD_NUMBER: _ClassVar[int]
    A2B_FAULT_FIELD_NUMBER: _ClassVar[int]
    a2b_slaves_discovered: int
    a2b_fault: StatusRespRoleA2BMaster.A2bFault
    def __init__(
        self,
        a2b_slaves_discovered: _Optional[int] = ...,
        a2b_fault: _Optional[_Union[StatusRespRoleA2BMaster.A2bFault, _Mapping]] = ...,
    ) -> None: ...

class StatusRespRoleA2BSlave(_message.Message):
    __slots__ = ("a2b_state",)
    A2B_STATE_FIELD_NUMBER: _ClassVar[int]
    a2b_state: SlaveA2BState
    def __init__(
        self, a2b_state: _Optional[_Union[SlaveA2BState, str]] = ...
    ) -> None: ...

class StatusResponse(_message.Message):
    __slots__ = (
        "usb_audio_downstream_state",
        "usb_audio_upstream_state",
        "device_state",
        "config_json_state",
        "status_master",
        "status_slave",
    )
    USB_AUDIO_DOWNSTREAM_STATE_FIELD_NUMBER: _ClassVar[int]
    USB_AUDIO_UPSTREAM_STATE_FIELD_NUMBER: _ClassVar[int]
    DEVICE_STATE_FIELD_NUMBER: _ClassVar[int]
    CONFIG_JSON_STATE_FIELD_NUMBER: _ClassVar[int]
    STATUS_MASTER_FIELD_NUMBER: _ClassVar[int]
    STATUS_SLAVE_FIELD_NUMBER: _ClassVar[int]
    usb_audio_downstream_state: UsbAudioStreamState
    usb_audio_upstream_state: UsbAudioStreamState
    device_state: DeviceState
    config_json_state: ConfigJsonState
    status_master: StatusRespRoleA2BMaster
    status_slave: StatusRespRoleA2BSlave
    def __init__(
        self,
        usb_audio_downstream_state: _Optional[_Union[UsbAudioStreamState, str]] = ...,
        usb_audio_upstream_state: _Optional[_Union[UsbAudioStreamState, str]] = ...,
        device_state: _Optional[_Union[DeviceState, str]] = ...,
        config_json_state: _Optional[_Union[ConfigJsonState, str]] = ...,
        status_master: _Optional[_Union[StatusRespRoleA2BMaster, _Mapping]] = ...,
        status_slave: _Optional[_Union[StatusRespRoleA2BSlave, _Mapping]] = ...,
    ) -> None: ...

class InfoResponse(_message.Message):
    __slots__ = ("hardware_revision", "software_revision", "serial_number")
    HARDWARE_REVISION_FIELD_NUMBER: _ClassVar[int]
    SOFTWARE_REVISION_FIELD_NUMBER: _ClassVar[int]
    SERIAL_NUMBER_FIELD_NUMBER: _ClassVar[int]
    hardware_revision: int
    software_revision: str
    serial_number: str
    def __init__(
        self,
        hardware_revision: _Optional[int] = ...,
        software_revision: _Optional[str] = ...,
        serial_number: _Optional[str] = ...,
    ) -> None: ...

class PositiveResponse(_message.Message):
    __slots__ = (
        "no_data_response",
        "status_response",
        "info_response",
        "i2c_over_distance_response",
        "a2b_mailbox_transfer_response",
    )
    NO_DATA_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    STATUS_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    INFO_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    I2C_OVER_DISTANCE_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    A2B_MAILBOX_TRANSFER_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    no_data_response: NoDataResponse
    status_response: StatusResponse
    info_response: InfoResponse
    i2c_over_distance_response: I2COverDistanceResponse
    a2b_mailbox_transfer_response: A2BMailboxTransferResponse
    def __init__(
        self,
        no_data_response: _Optional[_Union[NoDataResponse, _Mapping]] = ...,
        status_response: _Optional[_Union[StatusResponse, _Mapping]] = ...,
        info_response: _Optional[_Union[InfoResponse, _Mapping]] = ...,
        i2c_over_distance_response: _Optional[
            _Union[I2COverDistanceResponse, _Mapping]
        ] = ...,
        a2b_mailbox_transfer_response: _Optional[
            _Union[A2BMailboxTransferResponse, _Mapping]
        ] = ...,
    ) -> None: ...

class NegativeResponse(_message.Message):
    __slots__ = ("no_data", "text_error")
    NO_DATA_FIELD_NUMBER: _ClassVar[int]
    TEXT_ERROR_FIELD_NUMBER: _ClassVar[int]
    no_data: bool
    text_error: str
    def __init__(
        self, no_data: bool = ..., text_error: _Optional[str] = ...
    ) -> None: ...

class RequestPacket(_message.Message):
    __slots__ = (
        "reset_request",
        "a2b_discover_request",
        "status_request",
        "info_request",
        "set_serial_request",
        "i2c_over_distance_request",
        "a2b_mailbox_transfer_request",
    )
    RESET_REQUEST_FIELD_NUMBER: _ClassVar[int]
    A2B_DISCOVER_REQUEST_FIELD_NUMBER: _ClassVar[int]
    STATUS_REQUEST_FIELD_NUMBER: _ClassVar[int]
    INFO_REQUEST_FIELD_NUMBER: _ClassVar[int]
    SET_SERIAL_REQUEST_FIELD_NUMBER: _ClassVar[int]
    I2C_OVER_DISTANCE_REQUEST_FIELD_NUMBER: _ClassVar[int]
    A2B_MAILBOX_TRANSFER_REQUEST_FIELD_NUMBER: _ClassVar[int]
    reset_request: ResetRequest
    a2b_discover_request: A2BDiscoverRequest
    status_request: StatusRequest
    info_request: InfoRequest
    set_serial_request: SetSerialRequest
    i2c_over_distance_request: I2COverDistanceRequest
    a2b_mailbox_transfer_request: A2BMailboxTransferRequest
    def __init__(
        self,
        reset_request: _Optional[_Union[ResetRequest, _Mapping]] = ...,
        a2b_discover_request: _Optional[_Union[A2BDiscoverRequest, _Mapping]] = ...,
        status_request: _Optional[_Union[StatusRequest, _Mapping]] = ...,
        info_request: _Optional[_Union[InfoRequest, _Mapping]] = ...,
        set_serial_request: _Optional[_Union[SetSerialRequest, _Mapping]] = ...,
        i2c_over_distance_request: _Optional[
            _Union[I2COverDistanceRequest, _Mapping]
        ] = ...,
        a2b_mailbox_transfer_request: _Optional[
            _Union[A2BMailboxTransferRequest, _Mapping]
        ] = ...,
    ) -> None: ...

class ResponsePacket(_message.Message):
    __slots__ = ("positive_response", "negative_response")
    POSITIVE_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    NEGATIVE_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    positive_response: PositiveResponse
    negative_response: NegativeResponse
    def __init__(
        self,
        positive_response: _Optional[_Union[PositiveResponse, _Mapping]] = ...,
        negative_response: _Optional[_Union[NegativeResponse, _Mapping]] = ...,
    ) -> None: ...
